from .node_csv_parser import parse_node_csv
from .tpc_time_series_result_item import TPCResultItem
from ..abc.nodes import Nodes


class TPCNodes(TPCResultItem, Nodes):
    """TPC Nodes class."""

    def __repr__(self) -> str:
        if hasattr(self, 'fpath'):
            return f'<TPC Nodes: {self.fpath.stem}>'
        return '<TPC Nodes>'

    def load(self) -> None:
        # docstring inherited
        try:
            self.df = parse_node_csv(self.fpath)
        except Exception as e:
            raise Exception(f'Error loading TPC 1d_Nodes.csv file: {e}')

    def long_plot_result_types(self) -> list[str]:
        """Return list of result types for long plots.

        Returns
        -------
        list[str]
            List of result types for long plots.
        """
        result_types = ['Bed Level', 'Pit Level', 'Pipes'] + self.result_types(None)
        if self.maximums is not None and self.maximums.df is not None:
            maxes = [x for x in self.maximums.df.columns if 'TMax' not in x]
            result_types.extend(maxes)
        return result_types
